@@ -228,12 +228,10 @@ g_mime_charset_name (const char *charset)
 		
 		iso = strtoul (buf, &p, 10);
 		
-		g_assert (p > buf);
-		
 		if (iso == 10646) {
 			/* they all become ICONV_10646 */
 			iconv_name = g_strdup (ICONV_10646);
-		} else {
+		} else if (p > buf) {
 			buf = p;
 			if (*buf == '-' || *buf == '_')
 				buf++;
@@ -254,6 +252,10 @@ g_mime_charset_name (const char *charset)
 				iconv_name = g_strdup_printf (ICONV_ISO_S_FORMAT,
 							      iso, p);
 			}
+		} else {
+			/* p == buf, which probably means we've
+			   encountered an invalid iso charset name */
+			iconv_name = g_strdup (name);
 		}
 	} else if (!strncmp (name, "windows-", 8)) {
 		buf = name + 8;
@@ -396,8 +398,9 @@ g_mime_charset_best (const char *in, size_t inlen)
 #include <iconv.h>
 
 static struct {
-	char *name;
-	unsigned int bit;	/* assigned bit */
+	char *name;        /* charset name */
+	int multibyte;     /* charset type */
+	unsigned int bit;  /* assigned bit */
 } tables[] = {
 	/* These are the 8bit character sets (other than iso-8859-1,
 	 * which is special-cased) which are supported by both other
@@ -405,44 +408,34 @@ static struct {
 	 * they're listed in is the order they'll be tried in, so put
 	 * the more-popular ones first.
 	 */
-	{ "iso-8859-2", 0 },	/* Central/Eastern European */
-	{ "iso-8859-4", 0 },	/* Baltic */
-	{ "koi8-r", 0 },	/* Russian */
-	{ "koi8-u", 0 },	/* Ukranian */
-	{ "iso-8859-5", 0 },	/* Least-popular Russian encoding */
-	{ "iso-8859-7", 0 },	/* Greek */
-	{ "iso-8859-8", 0 },    /* Hebrew; Visual */
-	{ "iso-8859-9", 0 },	/* Turkish */
-	{ "iso-8859-13", 0 },	/* Baltic again */
-	{ "iso-8859-15", 0 },	/* New-and-improved iso-8859-1, but most
-				 * programs that support this support UTF8
-				 */
-	{ "windows-1251", 0 },	/* Russian */
-	{ 0, 0 }
-};
-
-/* Multibyte charsets - files are generated with gen-multibyte.c */
-static struct {
-	char *name;
-	char *filename;
-	unsigned int bit;
-} multibyte_tables[] = {
-	/* Japanese - in order of preference */
-	{ "iso-2022-jp", "iso-2022-jp.dat", 0 },
-	{ "Shift-JIS", "Shift-JIS.dat", 0 },
-	{ "euc-jp", "euc-jp.dat", 0 },
+	{ "iso-8859-2",   0, 0 },  /* Central/Eastern European */
+	{ "iso-8859-4",   0, 0 },  /* Baltic */
+	{ "koi8-r",       0, 0 },  /* Russian */
+	{ "koi8-u",       0, 0 },  /* Ukranian */
+	{ "iso-8859-5",   0, 0 },  /* Least-popular Russian encoding */
+	{ "iso-8859-7",   0, 0 },  /* Greek */
+	{ "iso-8859-8",   0, 0 },  /* Hebrew; Visual */
+	{ "iso-8859-9",   0, 0 },  /* Turkish */
+	{ "iso-8859-13",  0, 0 },  /* Baltic again */
+	{ "iso-8859-15",  0, 0 },  /* New-and-improved iso-8859-1, but most
+				    * programs that support this support UTF8
+				    */
+	{ "windows-1251", 0, 0 },  /* Russian */
 	
-	/* Korean - in order of preference */
-	{ "euc-kr", "euc-kr.dat", 0 },
-	{ "iso-2022-kr", "iso-2022-kr.dat", 0 },
-	
-	/* Simplified Chinese */
-	{ "gb2312", "gb2312.dat", 0 },
-	
-	/* Traditional Chinese - in order of preference */
-	{ "Big5", "Big5.dat", 0 },
-	{ "euc-tw", "euc-tw.dat", 0 },
-	{ NULL, NULL, 0 }
+	/* These are the multibyte character sets which are commonly
+	 * supported by other mail clients. Note: order for multibyte
+	 * charsets does not affect priority unlike the 8bit charsets
+	 * listed above.
+	 */
+	{ "iso-2022-jp",  1, 0 },  /* Japanese designed for use over the Net */
+	{ "Shift-JIS",    1, 0 },  /* Japanese as used by Windows and MacOS systems */
+	{ "euc-jp",       1, 0 },  /* Japanese traditionally used on Unix systems */
+	{ "euc-kr",       1, 0 },  /* Korean */
+	{ "iso-2022-kr",  1, 0 },  /* Korean (less popular than euc-kr) */
+	{ "gb2312",       1, 0 },  /* Simplified Chinese */
+	{ "Big5",         1, 0 },  /* Traditional Chinese */
+	{ "euc-tw",       1, 0 },
+	{ NULL, 0, 0}
 };
 
 unsigned int encoding_map[256 * 256];
@@ -455,12 +448,13 @@ unsigned int encoding_map[256 * 256];
 
 int main (int argc, char **argv)
 {
-	char *inptr, *outptr;
+	GHashTable *table_hash;
 	size_t inleft, outleft;
-	guint32 out[128];
+	char *inbuf, *outbuf;
+	guint32 out[128], c;
+	unsigned int bit = 0x01;
 	char in[128];
 	int i, j, k;
-	int bit = 0x01;
 	int bytes;
 	iconv_t cd;
 	
@@ -470,15 +464,15 @@ int main (int argc, char **argv)
 	for (i = 0; i < 128; i++)
 		in[i] = i + 128;
 	
-	for (j = 0; tables[j].name; j++) {
+	for (j = 0; tables[j].name && !tables[j].multibyte; j++) {
 		cd = iconv_open (UCS, tables[j].name);
-		inptr = in;
-		outptr = (char *)(out);
+		inbuf = in;
+		outbuf = (char *)(out);
 		inleft = sizeof (in);
 		outleft = sizeof (out);
-		while (iconv (cd, &inptr, &inleft, &outptr, &outleft) == -1) {
+		while (iconv (cd, &inbuf, &inleft, &outbuf, &outleft) == -1) {
 			if (errno == EILSEQ) {
-				inptr++;
+				inbuf++;
 				inleft--;
 			} else {
 				g_warning ("iconv (%s->UCS4, ..., %d, ..., %d):%s",
@@ -499,67 +493,74 @@ int main (int argc, char **argv)
 	}
 	
 	/* Mutibyte tables */
-	for (j = 0; multibyte_tables[j].name; j++) {
-		char *inbuf, *outbuf;
-		struct stat st;
-		unichar *c;
-		FILE *fp;
-		
-		if (stat (multibyte_tables[j].filename, &st) == -1)
+	for ( ; tables[j].name && tables[j].multibyte; j++) {
+		cd = iconv_open (tables[j].name, UCS);
+		if (cd == (iconv_t) -1)
 			continue;
 		
-		fp = fopen (multibyte_tables[j].filename, "r");
-		if (fp == NULL)
-			continue;
-		
-		inleft = st.st_size;
-		inbuf = g_malloc (st.st_size);
-		outleft = st.st_size * 6 + 20;
-		outptr = outbuf = g_malloc (outleft);
-		
-		fread (inbuf, 1, st.st_size, fp);
-		fclose (fp);
-		
-		cd = iconv_open (UCS, multibyte_tables[j].name);
-		
-		inptr = inbuf;
-		while (iconv (cd, &inptr, &inleft, &outptr, &outleft) == (size_t) -1) {
-			if (errno == EILSEQ || errno == EINVAL) {
-				inptr++;
-				inleft--;
+		for (c = 128, i = 0; c < 65535 && i < 65535; c++) {
+			inbuf = (char *) &c;
+			inleft = sizeof (c);
+			outbuf = in;
+			outleft = sizeof (in);
+			
+			if (iconv (cd, &inbuf, &inleft, &outbuf, &outleft) != (size_t) -1) {
+				/* this is a legal character in charset table[j].name */
+				iconv (cd, NULL, NULL, &outbuf, &outleft);
+				encoding_map[i++] |= bit;
+				encoding_map[c] |= bit;
 			} else {
-				g_warning ("iconv (%s->UCS4, ..., %d, ..., %d): %s\n",
-					   multibyte_tables[j].name, inleft, outleft,
-					   g_strerror (errno));
-				exit (1);
+				/* reset the iconv descriptor */
+				iconv (cd, NULL, NULL, NULL, NULL);
 			}
 		}
 		
 		iconv_close (cd);
 		
-		g_free (inbuf);
-		
-		for (i = 0, c = (unichar *) outbuf; (char *) c < outptr && *c < 65535 && i < 65535; c++) {
-			encoding_map[i++] |= bit;
-			encoding_map[*c] |= bit;
-		}
-		
-		g_free (outbuf);
-		
-		multibyte_tables[j].bit = bit;
+		tables[j].bit = bit;
 		bit <<= 1;
 	}
 	
 	printf ("/* This file is automatically generated: DO NOT EDIT */\n\n");
 	
+	/* FIXME: we can condense better than what my quick hack does,
+	   but it'd be more work and I'm not sure if it's worth it or
+	   not. Currently I'm just making it so that tables that
+	   contain all of the same values will only ever be
+	   one-of-a-kind by making duplicates into macro aliases for
+	   the original */
+	
+	table_hash = g_hash_table_new (g_int_hash, g_int_equal);
+	
 	for (i = 0; i < 256; i++) {
 		/* first, do we need this block? */
 		for (k = 0; k < bytes; k++) {
+			int first = encoding_map[i * 256] & (0xff << (k * 8));
+			int same = TRUE;
+			int dump = FALSE;
+			
 			for (j = 0; j < 256; j++) {
+				same = same && (encoding_map[i * 256 + j] & (0xff << (k * 8))) == first;
 				if ((encoding_map[i * 256 + j] & (0xff << (k * 8))) != 0)
-					break;
+					dump = TRUE;
 			}
-			if (j < 256) {
+			
+			if (dump) {
+				if (same) {
+					/* this table is aliasable */
+					char *table_name;
+					
+					if ((table_name = g_hash_table_lookup (table_hash, &first))) {
+						/* we've already written out a table with the exact same
+						   values so we can just alias it with a macro. */
+						printf ("#define m%02x%x %s\n\n", i, k, table_name);
+						continue;
+					} else {
+						table_name = g_strdup_printf ("m%02x%x", i, k);
+						g_hash_table_insert (table_hash, &first, table_name);
+					}
+				}
+				
 				/* yes, dump it */
 				printf ("static unsigned char m%02x%x[256] = {\n\t", i, k);
 				for (j = 0; j < 256; j++) {
@@ -604,9 +605,6 @@ int main (int argc, char **argv)
 	for (j = 0; tables[j].name; j++) {
 		printf ("\t{ \"%s\", 0x%04x },\n", tables[j].name, tables[j].bit);
 	}
-	for (j = 0; multibyte_tables[j].name; j++) {
-		printf ("\t{ \"%s\", 0x%04x },\n", multibyte_tables[j].name, multibyte_tables[j].bit);
-	}
 	printf ("};\n\n");
 	
 	printf("#define charset_mask(x) \\\n");
