@@ -21,24 +21,33 @@
  */
 
 #include <stdio.h>
+#include <string.h>
 #include <glib.h>
 
 #include "gmime.h"
 
+#define ENABLE_ZENTIMER
+#include "zentimer.h"
+
 void
-test_parser (gchar *data)
+test_parser (char *data)
 {
 	GMimeMessage *message;
-	gchar *text;
+	GMimeStream *stream;
+	char *text;
 	
 	fprintf (stdout, "\nTesting MIME parser...\n\n");
-	message = g_mime_parser_construct_message (data, strlen (data), TRUE);
+	
+	stream = g_mime_stream_mem_new_with_buffer (data, strlen (data));
+	message = g_mime_parser_construct_message (stream);
+	g_mime_stream_unref (stream);
+	
 	fprintf (stdout, "Test of GMimeHeader:\nTo: %s\n\n",
 		 g_mime_message_get_header (message, "To"));
 	text = g_mime_message_to_string (message);
 	fprintf (stdout, "Result should match previous MIME message dump\n\n%s\n", text);
 	g_free (text);
-	g_mime_message_destroy (message);
+	g_mime_object_unref (GMIME_OBJECT (message));
 }
 
 void
@@ -47,8 +56,7 @@ test_multipart (void)
 	GMimeMessage *message;
 	GMimeContentType *mime_type;
 	GMimePart *multi_part, *text_part, *html_part;
-	gchar *body;
-	gchar *text;
+	char *body, *text;
 	gboolean is_html;
 	
 	multi_part = g_mime_part_new_with_type ("multipart", "alternative");
@@ -58,6 +66,8 @@ test_multipart (void)
 	mime_type = g_mime_content_type_new ("text", "plain");
 	g_mime_part_set_content_type (text_part, mime_type);
 	g_mime_part_set_content_id (text_part, "1");
+	g_mime_part_set_content_header (text_part, "Content-Test", "test-mime");
+	g_mime_part_set_content_description (text_part, "this is a text part");
 	g_mime_part_set_content (text_part, "This is the body of my message.\n",
 				 strlen ("This is the body of my message.\n"));
 	
@@ -66,15 +76,16 @@ test_multipart (void)
 	html_part = g_mime_part_new ();
 	mime_type = g_mime_content_type_new ("text", "html");
 	g_mime_part_set_content_type (html_part, mime_type);
+	g_mime_part_set_filename (html_part, "this is a really really long filename that should force gmime to rfc2184 encode it - yay!.html");
 	g_mime_part_set_content_description (html_part, "this is an html part and stuff");
-	g_mime_part_set_content_id (html_part, "2");
 	g_mime_part_set_content (html_part, "<html>\n\t<pre>This is the body of my message.</pre>\n</html>",
 				 strlen ("<html>\n\t<pre>This is the body of my message.</pre>\n</html>"));
 	g_mime_part_set_encoding (html_part, GMIME_PART_ENCODING_BASE64);
+	g_mime_part_set_content_id (html_part, "2");
 	
 	g_mime_part_add_subpart (multi_part, html_part);
 	
-	message = g_mime_message_new ();
+	message = g_mime_message_new (TRUE);
 	g_mime_message_set_sender (message, "\"Jeffrey Stedfast\" <fejj@helixcode.com>");
 	g_mime_message_set_reply_to (message, "fejj@helixcode.com");
 	g_mime_message_add_recipient (message, GMIME_RECIPIENT_TYPE_TO,
@@ -99,7 +110,7 @@ test_multipart (void)
 	if (is_html)
 		fprintf (stdout, "yep...got it in html format\n");
 	
-	g_mime_message_destroy (message);
+	g_mime_object_unref (GMIME_OBJECT (message));
 	
 	test_parser (text);
 	
@@ -111,8 +122,7 @@ test_onepart (void)
 {
 	GMimeMessage *message;
 	GMimePart *mime_part;
-	gchar *body;
-	gchar *text;
+	char *body, *text;
 	gboolean is_html;
 	
 	mime_part = g_mime_part_new_with_type ("text", "plain");
@@ -120,7 +130,7 @@ test_onepart (void)
 	g_mime_part_set_content (mime_part, "This is the body of my message.\n",
 				 strlen ("This is the body of my message.\n"));
 	
-	message = g_mime_message_new ();
+	message = g_mime_message_new (TRUE);
 	g_mime_message_set_sender (message, "\"Jeffrey Stedfast\" <fejj@helixcode.com>");
 	g_mime_message_set_reply_to (message, "fejj@helixcode.com");
 	g_mime_message_add_recipient (message, GMIME_RECIPIENT_TYPE_TO,
@@ -137,7 +147,7 @@ test_onepart (void)
 	fprintf (stdout, "Trying to get message body:\n%s\n\n", body ? body : "(null)");
 	g_free (body);
 	
-	g_mime_message_destroy (message);
+	g_mime_object_unref (GMIME_OBJECT (message));
 	
 	test_parser (text);
 	
@@ -160,6 +170,47 @@ test_encodings (void)
 	char *enc, *dec;
 	int pos, state = -1, save = 0;
 	
+	enc = g_strdup ("OT - ich =?ISO-8859-1?Q?wei=DF?=, trotzdem");
+	fprintf (stderr, "encoded: %s\n", enc);
+	dec = g_mime_utils_8bit_header_decode (enc);
+	fprintf (stderr, "decoded: %s\n", dec);
+	g_free (enc);
+	g_free (dec);
+	
+	enc = g_strdup ("OT - ich =?iso-8859-1?b?d2Vp3yw=?= trotzdem");
+	fprintf (stderr, "encoded: %s\n", enc);
+	dec = g_mime_utils_8bit_header_decode (enc);
+	fprintf (stderr, "decoded: %s\n", dec);
+	g_free (enc);
+	g_free (dec);
+	
+	enc = g_strdup ("OT - ich =?ISO-8859-1?Q?wei=DF,?= trotzdem");
+	fprintf (stderr, "encoded: %s\n", enc);
+	dec = g_mime_utils_8bit_header_decode (enc);
+	fprintf (stderr, "decoded: %s\n", dec);
+	g_free (enc);
+	g_free (dec);
+	
+	enc = g_mime_utils_8bit_header_encode ("OT - ich wei, trotzdem");
+	fprintf (stderr, "encoded: %s\n", enc);
+	dec = g_mime_utils_8bit_header_decode (enc);
+	fprintf (stderr, "decoded: %s\n", dec);
+	g_free (enc);
+	g_free (dec);
+	
+	enc = g_strdup ("=?iso-8859-1?Q?=DE?=:\t=?iso-8859-1?Q?=AD=BE=EF?= spells 0xdeadbeef");
+	fprintf (stderr, "incorrect: %s\n", enc);
+	dec = g_mime_utils_8bit_header_decode (enc);
+	fprintf (stderr, "decoded: %s\n", dec);
+	g_free (enc);
+	enc = g_mime_utils_8bit_header_encode (dec);
+	g_free (dec);
+	fprintf (stderr, "correct: %s\n", enc);
+	dec = g_mime_utils_8bit_header_decode (enc);
+	fprintf (stderr, "decoded: %s\n", dec);
+	g_free (enc);
+	g_free (dec);
+	
 	enc = g_strdup ("=?iso-8859-1?q?blablah?=");
 	fprintf (stderr, "encoded: %s\n", enc);
 	dec = g_mime_utils_8bit_header_decode (enc);
@@ -237,7 +288,7 @@ test_encodings (void)
 	g_free (enc);
 }
 
-static gchar *addresses[] = {
+static char *addresses[] = {
 	"fejj@helixcode.com",
 	"Jeffrey Stedfast <fejj@helixcode.com>",
 	"Jeffrey \"fejj\" Stedfast <fejj@helixcode.com>",
@@ -259,20 +310,21 @@ static gchar *addresses[] = {
 	"<charles@>",
 	"<charles@broken.host.com.> (Charles Kerr)",
 	"fpons@mandrakesoft.com (=?iso-8859-1?q?Fran=E7ois?= Pons likes _'s and \t's too)",
+	"Tivo Leedjrv <leedjarv@interest.ee>",
 	NULL
 };
 
 static void
-dump_addrlist (GList *addrlist, int i, gboolean group, gboolean destroy)
+dump_addrlist (InternetAddressList *addrlist, int i, gboolean group, gboolean destroy)
 {
+	InternetAddressList *addr;
 	InternetAddress *ia;
-	GList *addr;
 	
 	addr = addrlist;
 	while (addr) {
 		char *str;
 		
-		ia = addr->data;
+		ia = addr->address;
 		addr = addr->next;
 		
 		if (i != -1)
@@ -298,9 +350,6 @@ dump_addrlist (GList *addrlist, int i, gboolean group, gboolean destroy)
 		fprintf (stderr, "%sRewritten (encoded): %s\n\n", group ? "\t" : "",
 			 str ? str : "");
 		g_free (str);
-		
-		if (destroy)
-			internet_address_destroy (ia);
 	}
 }
 
@@ -310,7 +359,7 @@ test_addresses (void)
 	int i;
 	
 	for (i = 0; addresses[i]; i++) {
-		GList *addrlist;
+		InternetAddressList *addrlist;
 		
 		addrlist = internet_address_parse_string (addresses[i]);
 		if (!addrlist) {
@@ -319,37 +368,64 @@ test_addresses (void)
 		}
 		
 		dump_addrlist (addrlist, i, FALSE, TRUE);
+		
+		internet_address_list_destroy (addrlist);
 	}
 }
 
+/*#include "date-strings.h"*/
+
 void
 test_date (void)
 {
-	gint offset = 0;
-	gchar * in = NULL;
-	gchar * out = NULL;
-	time_t date = 0;
-
+	int offset = 0;
+	char *in, *out;
+	time_t date;
+	
 	in = "Mon, 17 Jan 1994 11:14:55 -0500";
 	fprintf (stderr, "date  in: [%s]\n", in);
 	date = g_mime_utils_header_decode_date (in, &offset);
 	out = g_mime_utils_header_format_date (date, offset);
 	fprintf (stderr, "date out: [%s]\n", out);
 	g_free (out);
-
+	
 	in = "Mon, 17 Jan 01 11:14:55 -0500";
 	fprintf (stderr, "date  in: [%s]\n", in);
 	date = g_mime_utils_header_decode_date (in, &offset);
 	out = g_mime_utils_header_format_date (date, offset);
 	fprintf (stderr, "date out: [%s]\n", out);
 	g_free (out);
+
+#if 0
+	{
+		char *string;
+		int i;
+		
+		ZenTimerStart ();
+		for (i = 0; i < num_date_strings; i++) {
+			in = date_strings[i];
+			printf ("date  in: [%s]\n", in);
+			date = g_mime_utils_header_decode_date (in, &offset);
+			out = g_mime_utils_header_format_date (date, offset);
+			printf ("date out: [%s]\n", out);
+			g_free (out);
+		}
+		ZenTimerStop ();
+		
+		string = g_strdup_printf ("date parser (%d dates parsed)", num_date_strings);
+		ZenTimerReport (string);
+		g_free (string);
+	}
+#endif
 }
 
 
 int main (int argc, char *argv[])
 {
+	g_mime_init (0 /*GMIME_INIT_FLAG_UTF8*/);
+	
 	test_date ();
-
+	
 	test_onepart ();
 	
 	test_multipart ();
